﻿// ************************** Script Information **************************
// RegExColumns V2.0
// (tested with Directory Opus 11.7.5)
// © 2014 /^\\iran
// 
// ****************************** Change log ******************************
// 2014-07-16 V1.0		Initial Release
// 2014-07-17 V1.1		Added Multi capture group for the RegExp
// 2014-07-17 V1.2		Updated to the DOpus V11.5.2b changes 
// 2014-07-25 V1.3		Addes Column Type "star"
// 2014-07-28 V1.3.1	Some fixes for column type "star"
// 2014-11-09 V2.0		Column Config moved from script to DOpus Script Config
//
// ************************************************************************
// *                             Script-Data                              *
// ************************************************************************

SCRIPT_NAME = "RegExColumns";
SCRIPT_VERSION = "V2.0";
SCRIPT_DATE = "09.11.2104";
COPYRIGHT = "© 2014 /^\\iran";
MIN_VERSION = "11.7";

//var RegExColumns = {};
	
// ************************************************************************
// *                              Functions                               *
// ************************************************************************

// Called by Directory Opus to initialize the script
function OnInit(initData)
{
	// DOpus Version Check 
	if(!DOpus.version.atLeast(MIN_VERSION)){DOpus.OutputString("ERR: DOpus Version " + MIN_VERSION + "required to run this script!")}
   
	// Provide basic information about the Script
	initData.name = SCRIPT_NAME;
	initData.desc = "Create Columns from Filename with Regular Expressions.";
	initData.copyright = COPYRIGHT;
	initData.min_version = MIN_VERSION;
	initData.default_enable = true;
	
	// Create Map for Config Descriptions
	var ConfigDescriptions = DOpus.Create.Map();
		
	// Configuration Options of the Script
	initData.config.LogLevel = vecLogLevel;
	ConfigDescriptions("LogLevel") = "Defines the details of log information the script outputs to the dopus other log.\nDefault: WARNING\n";
	 
	// RegExColumns Examples
	RegExColumns =  '"RegExColExample - Number": {"header":"Number", "inputItemProperty": "name", "regexp":"^(..) Number.*$", "type": "number"},\r\n';
	RegExColumns += '"RegExColExample - Double": {"header":"Double", "inputItemProperty": "name", "regexp":"^(....) Double.*$", "type": "double", "justify": "right"},\r\n';
	RegExColumns += '"RegExColExample - Percent":{"header":"Percent","inputItemProperty": "name", "regexp":"^(..) Percent.*$", "type": "percent"},\r\n';
	RegExColumns += '"RegExColExample - Graph":  {"header":"Graph",  "inputItemProperty": "name", "regexp":"^(..) Graph.*$", "type": "graph"},\r\n';
	RegExColumns += '"RegExColExample - Rating": {"header":"Rating", "inputItemProperty": "name", "regexp":"^(..) Rating.*$", "type": "star", "maxStars" : "10", "maxStarValue": "100"},\r\n';
	RegExColumns += '"RegExColExample - Size":   {"header":"Size",   "inputItemProperty": "name", "regexp":"^([0-9]*) Size.*$", "type": "size"},\r\n';
	RegExColumns += '"RegExColExample - Date":   {"header":"Date",   "inputItemProperty": "name", "regexp":"^([0-9]{4}-[0-9]{2}-[0-9]{2}) Date.*$", "type": "date", "datetimeformat":"yyyy-mm-dd"}\r\n';

	
	initData.config.RegExColumns = RegExColumns;
	ConfigDescriptions("RegExColumns") = 'RegEx Columns Definition in JSON Format.';
	// Add Configuration Descriptions
	initData.config_desc = ConfigDescriptions;
	return false;
}

function OnColumn(scriptColData)
{
	LogLevel = Script.config.LogLevel;	
//	Logger (LOG_DEBUG, ("LogLevelName: " + vecLogLevel(LogLevel+1)));

	RegExColumns = JSON.parse("{" + Script.config.RegExColumns + "}");

	Logger (LOG_DEBUG, "*** Enter Column: " + scriptColData.col);
	Logger (LOG_DEBUG, "RegExp: "+ RegExColumns[scriptColData.col].regexp);
	Logger (LOG_DEBUG, "File: " + scriptColData.item.name);
	
	// Start with empty value
	scriptColData.value = null;

	input = "" + scriptColData.item;
	if(RegExColumns[scriptColData.col].inputItemProperty)input = scriptColData.item[RegExColumns[scriptColData.col].inputItemProperty];
	Logger (LOG_DEBUG, "Input: " + input)
	
	// Use Regexp
	RegExpResult = input.match( new RegExp(RegExColumns[scriptColData.col].regexp));
	Logger (LOG_DEBUG, "RegExpResult: " + RegExpResult);

	// empty cell if no match
	if(RegExpResult == null)	
	{
		scriptColData.type = "text";
		scriptColData.value = "";
		return
	}
	
	resultValue = "";
		
	for (var i = 1; i < RegExpResult.length; i++)
	{
		resultValue += RegExpResult[i];
	}

	// Convert result if it is a date/datetime column
	if(/^date.*/i.test(RegExColumns[scriptColData.col].type))
	{
		resultValue = parseDate(resultValue, RegExColumns[scriptColData.col].datetimeformat).getVarDate();
	}

	// Add max stars if column type star and maxstars configured
	if(RegExColumns[scriptColData.col].type == "star" && RegExColumns[scriptColData.col].maxStars !== null)
	{
		if(RegExColumns[scriptColData.col].maxStarValue !== null)
			{resultValue = Math.round(resultValue / RegExColumns[scriptColData.col].maxStarValue * RegExColumns[scriptColData.col].maxStars)}		
	}
	
	
	Logger (LOG_DEBUG, "Result: " + resultValue);
	scriptColData.value = resultValue;
		
	Logger (LOG_DEBUG, "*** Exit Column: " + scriptColData.col);
	Logger (LOG_DEBUG, "");
}

function OnScriptConfigChange(data) {
	LogLevel = Script.config.LogLevel;	
	Logger (LOG_DEBUG, "Config Changed.");
	Logger (LOG_DEBUG, Script.config.RegExColumns);
	try {
		RegExColumns = JSON.parse("{" + Script.config.RegExColumns + "}");
	}
	catch(err) {
		Logger (LOG_ERR, "Error parsing RegExColums!");
		Logger (LOG_ERR, "  " + err);
		dialogRegExColumnsError(err);
		return;
	}
	Script.InitColumns();
}

function OnAddColumns(data) {
	LogLevel = Script.config.LogLevel;	
	Logger (LOG_DEBUG, "Add/Update Columns");
	
	try {
		RegExColumns = JSON.parse("{" + Script.config.RegExColumns + "}");
	}
	catch(err) {
		Logger (LOG_ERR, "Error parsing RegExColums!");
		Logger (LOG_ERR, "  " + err);
		return;
	}

	for(var ColumnName in RegExColumns)
	{
		var column = data.AddColumn();
		column.method = "OnColumn";
		column.autorefresh = false;
		column.namerefresh = true; 
		column.name = ColumnName;
		column.label = ColumnName;
		if(RegExColumns[ColumnName].header !== null) column.header = RegExColumns[ColumnName].header;
		if(RegExColumns[ColumnName].type !== null) column.type = RegExColumns[ColumnName].type;
		if(RegExColumns[ColumnName].nogroup !== null) column.nogroup = RegExColumns[ColumnName].nogroup;
		if(RegExColumns[ColumnName].nosort !== null) column.nosort = RegExColumns[ColumnName].nosort;
		if(RegExColumns[ColumnName].justify !== null) column.justify = RegExColumns[ColumnName].justify;
		if(RegExColumns[ColumnName].type == "star" && RegExColumns[ColumnName].maxStars !== null) column.maxstars = RegExColumns[ColumnName].maxStars;
	}
}

// ************************************************************************
// *                            About-Dialog                              *
// ************************************************************************

function OnAboutScript(aboutData)
{
	LogLevel = Script.config.LogLevel;	
	Logger (LOG_DEBUG, ("LogLevelName: " + vecLogLevel(LogLevel+1)));
	Logger (LOG_DEBUG, "*** Enter About");
		
	About = "";
	About += SCRIPT_NAME + " " + SCRIPT_VERSION +"\n";
	About += SCRIPT_DATE + "\n\n";
	About += COPYRIGHT + "\n\n";
	About += "Feel free to use, optimize and enhance the script. Please post your comments and changes in the respective thread of the Dopus Resource Centre forum.\n";

	Config = "";
	Config += "*** Configuration Parameters ***\n";
	Config += "\n";
	Config += "LOGLEVEL\n";
	Config += "Defines the details of log information the script outputs to the dopus other log.\n";
	Config += "Default: WARNING\n";
	Config += "\n";
	Config += "REGEXCOLUMNS\n";
	Config += "regexp: (mandatory)\n";
	Config += "Regular expression, value for the column is the concatenation of all capture groups.\n";
	Config += "\n";
	Config += "type:\n";
	Config += "Type for the column as described in the DOpus documentation.\n";
	Config += "\n";
	Config += "header:\n";
	Config += "Set the header for the column as described in the DOpus documentation.\n";
	Config += "\n";
	Config += "nogroup:\n";
	Config += "Flag (true/false) as described in the DOpus documentation.\n";
	Config += "\n";
	Config += "nosort:\n";
	Config += "Flag (true/false) as described in the DOpus documentation.\n";
	Config += "\n";
	Config += "justify:\n";
	Config += "Justification as described in the DOpus documentation.\n";
	Config += "\n";
	Config += "datetimeformat:\n";
	Config += "Date Format used to parse the extracted date information (time not realy supported at the moment!).\n";
	Config += "\n";
	Config += "inputItemProperty:\n";
	Config += "Input for the regexp from the Item-Object as described in the DOpus documentation (e.g. default: path plus filename; name = item name only).\n";
	Config += "\n";
	Config += "maxStars:\n";
	Config += "Maximum number of stars for star column type as described in the DOpus documentation.\n";
	Config += "\n";
	Config += "maxStarValue:\n";
	Config += "Maximum number for the value to skale to the maxStars.\n";
	Config += "\n";
	
	// Create Dialog
	dlg = DOpus.Dlg;
	dlg.window = aboutData.window;
	dlg.title = SCRIPT_NAME;
	dlg.buttons = "About|Configuration Parameter|Exit";
	dlg.icon = "info";
	
	ret = 1;
	do {
		if(ret == 1)	dlg.message = About;
		if(ret == 2)	dlg.message = Config;
		ret = dlg.show;
	}
	while (ret != 0);

	Logger (LOG_DEBUG, "*** Exit About");

	return false;
}

// ************************************************************************
// *                            About-Dialog                              *
// ************************************************************************

function dialogRegExColumnsError(err)
{
	dlg = DOpus.Dlg;
	dlg.message = "Error in RegExColumns Definition\n\n" + err;
	dlg.title = "RegEx Columns Error"
	dlg.buttons = "Ok";	
	dlg.icon = "error";
	dlg.show;
} 

// ************************************************************************
// *                          Utility Functions                           *
// ************************************************************************

// **********
// * Logger *
// **********
var LogLevel = LOG_WARN;
var LOG_DEBUG = 0;
var LOG_INFO = 1;
var LOG_WARN = 2;
var LOG_ERR = 3;
var LOG_ALERT = 4;

vecLogLevel = DOpus.NewVector;
	vecLogLevel.push_back(LOG_WARN);
	vecLogLevel.push_back("DEBUG");
	vecLogLevel.push_back("INFO");
	vecLogLevel.push_back("WARNING");
	vecLogLevel.push_back("ERROR");
	vecLogLevel.push_back("ALERT");

function Logger(Level, strMessage)
{
	if (Level>=LogLevel){DOpus.OutputString(strMessage);}
}

// *********************
// * Date/Time Parsing *
// *********************

function parseDate(input, format) {
  format = format || 'yyyy-mm-dd'; // default format
  var parts = input.match(/(\d+)/g), 
      i = 0, fmt = {};
  // extract date-part indexes from the format
  format.replace(/(yyyy|dd|mm)/g, function(part) { fmt[part] = i++; });

  return new Date(parts[fmt['yyyy']], parts[fmt['mm']]-1, parts[fmt['dd']]);
}